/******************************************************************************/
/* passive tissue server                                                      */
/* (c) 2006 jamie twycross, jpt AT cs.nott.ac.uk                              */
/* released under gnu gpl v2                                                  */
/******************************************************************************/

#include <tissue/tissue.h>

#include <stdlib.h>
#include <unistd.h>
#include <err.h>
#include <sys/time.h>
#include <time.h>
#include <stdio.h>

/* command line parameters */
static unsigned int max_antigen = 100;  /* max number of antigen in tissue */
static unsigned int max_cytokines = 10;  /* max number of cytokine in tissue */
static unsigned long output_rate = 1000000;  /* rate to print stats (usecs) */
static int16_t server_port = TS_DEFAULT_PORT;  /* port to listen on */
static short debug = 0;  /* extra info in tissue.log */

static void parse_command_line(int argc, char **argv)
{
	int c;

	while((c = getopt(argc, argv, "dp:a:c:r:")) != -1)
		switch(c) {
			case 'p':  /* tissue server port */
				server_port = (int16_t) atoi(optarg);
				break;

			case 'a':  /* max number of antigen in tissue */
				max_antigen = (unsigned int) atoi(optarg);
				break;

			case 'c':  /* max number of cytokines in tissue */
				max_cytokines = (unsigned int) atoi(optarg);
				break;

			case 'r':  /* rate to print stats (usecs) */
				output_rate = (unsigned long) atoi(optarg);
				break;

			case 'd':  /* debug (save detailed info to tissue.log */
				debug = 1;
				break;

			default:
			usage:
				errx(1, "usage: tissue-server [-d -p PORT -a NUM " \
					"-c NUM -r NUM]\n" \
					"\t-d - debug (more info in tissue.log\n" \
					"\t-p PORT - port server listens on\n" \
					"\t-a NUM - max anitgen in tissue\n" \
					"\t-c NUM - max cytokines in tissue\n" \
					"\t-r NUM - output rate (usecs)\n");
				break;
		}

	if(optind != argc)
		goto usage;
}

int main(int argc, char **argv)
{
	Tissue *tissue;
	struct timeval curtime;

	srand(time(NULL));
	srand48(time(NULL));

	parse_command_line(argc, argv);

	/* create some tissue - no cells */
	tissue = init_tissue(server_port, max_antigen, max_cytokines, 0);

	/* extra logging in tissue.log if requested */
	if(debug)
		tissue->log = 1;

	/* just print the tissue */
	while(1) {
		gettimeofday(&curtime, NULL);
		printf("%d.%d ", (int) curtime.tv_sec, (int) curtime.tv_usec);
		print_tissue_antigen(tissue);
		printf("%d.%d ", (int) curtime.tv_sec, (int) curtime.tv_usec);
		print_tissue_cytokines(tissue);
		usleep(output_rate);
	}

	return EXIT_SUCCESS;
}
